unit QTimers;

  { ****************************************************************** }
  {                                                                    }
  {           Delphi (6) unit  --   Timing utility                     }
  {                                                                    }
  {           Copyright  2004 by Olivier Touzot "QnnO"                }
  {         (http://mapage.noos.fr/qnno/  -  qnno@noos.fr)             }
  { v 1.0                                                              }
  { ****************************************************************** }

  //  This unit is freeware, but under copyrights that remain mine for my
  //  parts of the code, and original writters for their parts of the code.

  //  This unit can be freely used in any application, freeware, shareware
  //  or commercial. However, I would apreciate your sending me an email if
  //  you decide to use it. Of course, you use it under your own and single
  //  responsability. Neither me, nor contributors, could be held responsible
  //  for any problem resulting from the use of this unit.  ;-)

  //  It can also be freely distributed, provided all the above (and current)
  //  lines remain within it unchanged, and the TQTimer_HowTo.html file be
  //  distributed with it too.


  //  Usage : Once instanciated, you can start adding values in your timer ;
  //  ------  You can instanciate as many timers as you want, for example
  //          to specialize each one in a particular counter and/or time unit.
  //  For in-depth explanations, see the TQTimer_HowTo.html file that you
  //  downloaded with this one.
  

interface

uses SysUtils,   // IntToStr, format, time manipulation, aso.
     Windows,    // needed for GetTickCount, QueryPerfXXX, registry keys
     Math,       // needed for min. You can remove it if you write your own Min() function
     registry;   // needed for TRegistry type


const
  TimeScale2Str : Array[0..3] of String = ('s.', 'ms', 's', 'ns');
  TimerUnit2Str : Array[0..3] of String = ( 's.', 'ms.', '"units"', 'cycles' );

type
  TQTimeScale   = (tsSeconds, tsMilliSec, tsMicroSec, tsNanosSec);
  TQTimerAction = (taStart,taStop,taGlobalStop);
  TQTimerWanted = (twStart,twStop,twDelta);
  TQTimerKind   = (tkHMS,tkGTC,tkQPC,tkRDT);

  TQTimerEntry  = record
    tStart,
    tStop : int64;
    kind  : TQTimerKind;
  End;

  TQTimer = class
  Private
    FEntry : Array of TQTimerEntry;
    FIsFinalized : Boolean;
    FRegFrq,                           // Registry stored frequency  * 1000000
    FQpcFrq,                           // Returned value of QueryPerformanceFrequency
    FRDTOverhead,                      // direct ASM call overhead
    FQpcOverhead: int64;               // direct API call overhead
  //Internals
    Function  GetCount: Integer;
    Function  GetStartAsInt(ix:integer):Int64;
    Function  GetStopAsInt (ix:integer):Int64;
    Function  GetDeltaAsInt(ix:integer):Int64;
    Function  GetStartAsStr(ix:integer):String;
    Function  GetStopAsStr (ix:integer):String;
    Function  GetDeltaAsStr(ix:integer):String;
    Function  GetFormatedDeltaAsExt(ix:Integer):Extended;
    Function  GetFormatedDeltaAsStr(ix:Integer):String;
    Function  GetEntry(ix:Integer;wanted:TQTimerWanted):Int64;
    Function  Overhead(ix:Integer):integer;
    Function  GetTimeMeasureAs(timeScale:TQTimeScale; measure:Int64; counter:TQTimerKind):real;
    Function  GetOptimalMeasure(measure:Int64; counter:TQTimerKind; var tScale:TQTimeScale):real;
    Procedure Store(value:Int64;inOrOut:TQTimerAction;timerKind:TQTimerKind);
    Function  Int64ToTime(value:Int64):String;
    Procedure InitTimerEnvir;
  Public

    fmtOutput        : String;                    // Used for the Format() function ; Defaults to '%3.9n'
    timeUnitsWanted  : Set of TQTimeScale;
    removeOverHead,
    deltaShowNatives,                             // outputs deltas in native units;
    deltaShowOptimal : Boolean;                   // outputs deltas in the most "readable" unit possible;

    Constructor Create;
    Destructor  Destroy; override;

    Function  HMS(startOrStop:TQTimerAction=taStart):TDateTime;
    Function  GTC(startOrStop:TQTimerAction=taStart):Integer;
    Function  QPC(startOrStop:TQTimerAction=taStart):Int64;
    Function  RDT(startOrStop:TQTimerAction=taStart):Int64;
    Procedure GlobalStop;
    Procedure Reset;

    Property isFinalized        : Boolean  Read FIsFinalized;
    Property RegistryFreq       : int64    Read FRegFrq;
    Property QueryPerfFreq      : Int64    Read FQpcFrq;
    Property ReaDTSCOverhead    : int64    Read FRDTOverhead;
    Property QueryPerfOverhead  : int64    Read FQpcOverhead;

    Property Count              : integer  Read GetCount;
    Property iStart[ix:Integer] : int64    Read GetStartAsInt;
    Property iStop [ix:Integer] : int64    Read GetStopAsInt;
    Property iDelta[ix:Integer] : int64    Read GetDeltaAsInt;

    Property sStart[ix:Integer] : String   Read GetStartAsStr;
    Property sStop [ix:Integer] : String   Read GetStopAsStr;
    Property sDelta[ix:Integer] : String   Read GetDeltaAsStr;

    Property iDeltaFmt[ix:Integer] : Extended Read GetFormatedDeltaAsExt;
    Property sDeltaFmt[ix:Integer] : String   Read GetFormatedDeltaAsStr;
  End;


  //Not a member of the class :
   Function  RDTSC: Int64; assembler;


implementation


//Standallone
Function RDTSC: Int64; assembler;               //NIH...
asm
  Rdtsc                                         //<=> DB $0F,$31  Result in EAX and EDX (Int64)
end;


Constructor TQTimer.Create;
// Default output for timer deltas would give sthg like
// "12554092 units, 21.98452 ms"
Begin
  Inherited Create;
  fmtOutput        := '%3.9n';
  TimeUnitsWanted  := [];
  removeOverHead   := False;
  deltaShowNatives := True;                                // Affiche la valeur (Start, stop ou Delta) telle que stocke;
  deltaShowOptimal := True;
  FIsFinalized     := False;
  InitTimerEnvir;
End;

Destructor TQTimer.Destroy;
Begin
  If Count > 0 Then Reset;
  Inherited;
End;

Procedure TQTimer.Reset;
Begin
  SetLength(FEntry,0);
End;

Function TQTimer.GetCount:Integer;
Begin
  result := Length(FEntry);
End;

Function TQTimer.GetStartAsInt(ix:integer):Int64;
Begin
  result := GetEntry(ix,twStart);
End;

Function TQTimer.GetStopAsInt(ix:integer):Int64;
Begin
  result := GetEntry(ix,twStop);
End;

Function TQTimer.GetDeltaAsInt(ix:integer):Int64;
Begin
  result := GetEntry(ix,twDelta);
End;


Function TQTimer.GetEntry(ix:Integer;wanted:TQTimerWanted):Int64;
Begin
  result := -1;
  ix := Abs(ix);
  TRY
    If ix > (Count -1) Then exit
    Else Case wanted of
           twDelta : result := FEntry[ix].tStop - FEntry[ix].tStart - Overhead(ix);
           twStart : result := FEntry[ix].tStart;
         Else result := FEntry[ix].tStop;
         End;{case}
  EXCEPT ; END;
End;


Function TQTimer.Overhead(ix:Integer):integer;
Begin
  result := 0;
  If Not(FIsFinalized) or Not(removeOverHead) Then Exit;
  TRY
    Case FEntry[ix].kind of
      tkQPC : result := FQpcOverhead;
      tkRDT : result := FRDTOverhead;
    Else {hms, gtc : overhead=0};
    End; {case}
  EXCEPT;
  END;
End;


Function TQTimer.GetStartAsStr(ix:integer):String;
Begin
  result := IntToStr(GetStartAsInt(ix));
End;

Function TQTimer.GetStopAsStr(ix:integer):String;
Begin
  result := IntToStr(GetStopAsInt(ix));
End;

Function TQTimer.GetDeltaAsStr(ix:integer):String;
Begin
  result := IntToStr(GetDeltaAsInt(ix));
End;


Function TQTimer.GetFormatedDeltaAsExt(ix:Integer):Extended;
var dummy : TQTimeScale;
Begin
  result := 0.00;
  TRY
  If Not(FIsFinalized) or Not(deltaShowOptimal)
     Then result := ( GetDeltaAsInt(ix) / 1 )       // Makes it an extended...
     Else result := GetOptimalMeasure(GetDeltaAsInt(ix), FEntry[ix].kind, dummy);
  EXCEPT;
  END;
End;


Function TQTimer.GetFormatedDeltaAsStr(ix:Integer):String;
var tmScale  : TQTimeScale;
    split  : String;
    counter: TQTimerKind;
Begin
  result := '';
  split  := '';
  TRY
  If Not(FIsFinalized) Then result := GetDeltaAsStr(ix) + ' ' + TimerUnit2Str[ Ord(FEntry[ix].kind) ]
  Else Begin
         counter := FEntry[ix].kind;
         If counter = tkHMS Then    //hms is a special case
         Begin
           result := Int64ToTime(GetDeltaAsInt(ix));
           Exit;
         end;
         If deltaShowNatives Then
         Begin
           result := GetDeltaAsStr(ix) + ' ' + TimerUnit2Str[ Ord(counter) ];
           split  := ', ';
         End;
         If tsSeconds  In timeUnitsWanted Then
         Begin
           result := result + split + Format(fmtOutput,[getTimeMeasureAs(tsSeconds, GetDeltaAsInt(ix),counter)])
                                    + ' s.' ;     //+ TimeScale2Str[ord(tsSecondes)];
           split  := ', ';
         End;
         If tsMilliSec  In timeUnitsWanted Then
         Begin
           result := result + split + Format(fmtOutput,[getTimeMeasureAs(tsMilliSec,GetDeltaAsInt(ix),counter)])
                                    + ' ms';
           split  := ', ';
         End;
         If tsMicroSec  In timeUnitsWanted Then
         Begin
           result := result + split + Format(fmtOutput,[getTimeMeasureAs(tsMicroSec,GetDeltaAsInt(ix),counter)])
                                    + ' s';
           split  := ', ';
         End;
         If tsNanosSec   In timeUnitsWanted Then
         Begin
           result := result + split + Format(fmtOutput,[getTimeMeasureAs(tsNanosSec,GetDeltaAsInt(ix),counter)])
                                    + ' ns';
           split  := ', ';
         End;
         If deltaShowOptimal
            Then result := result + split + Format(fmtOutput,[GetOptimalMeasure(GetDeltaAsInt(ix),counter, tmScale)])
                                  + ' '   + TimeScale2Str[ord(tmScale)];
       End;
  EXCEPT;
  END;
End;


Procedure TQTimer.Store(value:Int64;inOrOut:TQTimerAction;timerKind:TQTimerKind);
Var ix:Integer;
Begin
  TRY
  Case inOrOut of
    taStop  :  For ix := (Length(FEntry)-1) DownTo 0 Do
                   If FEntry[ix].tStop = 0 Then
                      Begin
                        FEntry[ix].tStop := value;
                        break;
                      End;

    taStart : Begin
                ix := Length(FEntry);
                If (ix = Pred(High(Integer))) Then exit;
                SetLength(FEntry,ix+1);
                FEntry[ix].tStart := value;
                FEntry[ix].tStop := 0;
                FEntry[ix].kind := timerKind;
              End;
  end;{case}
  EXCEPT; END;
End;

Function TQTimer.Int64ToTime(value:Int64):String;
// Converts an int64 back to hms
var Hour, Min, Sec,
    MSec, rmdM : Word;
Begin
  TRY
    hour := value Div 3600;
    rmdM := value - (hour * 3600);
    Min  := rmdM div 60;
    Sec  := rmdM mod 60;
    Msec := 0;
    result := TimeToStr(EncodeTime(Hour, Min, Sec, MSec));
  EXCEPT result := IntToStr(value);
  END;
End;

Function  TQTimer.HMS(startOrStop:TQTimerAction=taStart):TDateTime;
var Hour, Min, Sec, MSec: Word;
    dte64 :Int64;
Begin
  result := Now;
  DecodeTime(result, Hour, Min, Sec, MSec);
  dte64 := ( (hour * 3600)
         +   (Min  * 60)
         +   (sec ) );
  Store(dte64,startOrStop,tkHMS);
End;

Function  TQTimer.GTC(startOrStop:TQTimerAction=taStart):Integer;
Begin
  result := GetTickCount;
  Store(Int64(result),startOrStop,tkGTC);
End;

Function  TQTimer.QPC(startOrStop:TQTimerAction=taStart):Int64;
Begin
TRY
  QueryPerformanceCounter(result);
  Store(result,startOrStop,tkQPC);
EXCEPT; result := -1; END;
End;

Function  TQTimer.RDT(startOrStop:TQTimerAction=taStart):Int64;
Begin
TRY
  result := RDTSC;
  Store(result,startOrStop,tkRDT);
EXCEPT; result := -1; END;
end;



Procedure TQTimer.GlobalStop;
// GlobalStop will read the first non-zero stop entry in the storage table,
// (starting from the end (ie. the most recent one)), and copy it in any
// not yet filled entry.
// If no stop has been emitted before, it does nothing...
// Speed is not an issue : We copy sthg that soon exists, we don't make a measure.

// globalStop is usefull if ALL stored entries not yet stopped come from the
// same counter ; Otherwise it is useless, and will output erroneous measures.

Var ix:Integer;
    gStop:Int64;
Begin
  gStop := 0;
  TRY
    For ix := (Length(FEntry)-1) downTo 0 Do
        If FEntry[ix].tStop <> 0 Then Begin
                                    gStop := FEntry[ix].tStop;
                                    break;
                                  End;
    If gStop <> 0 Then
       For ix := (Length(FEntry)-1) downTo 0 Do
           If FEntry[ix].tStop = 0 Then FEntry[ix].tStop := gStop;
  EXCEPT; END;
End;


Function TQTimer.GetTimeMeasureAs(timeScale:TQTimeScale;
                          measure:Int64;
                          counter:TQTimerKind):real;
// converts a reading (delta) in a time measure corresponding to the timescale wanted;
const scaler : Array[0..3] of int64  = (1,1000,1000000,1000000000);
//                        (tsSecondes, tsMilliSec, tsMicroSec, tsNanosSec);
Begin
  Result := measure/1;
  If Not(FIsFinalized) then Exit;
  TRY
  Case counter of
    tkQPC : Begin
              If FQpcFrq > 0 Then
                 result := (measure / FQpcFrq) * scaler[ord(timeScale)];
            End;
    tkRDT : Begin
              If FRegFrq > 0 Then
                 result := (measure / FRegFrq) * scaler[ord(timeScale)];
            End;
    tkGTC : Begin
              If FRegFrq > 0 Then
                 result := (measure / 1000) * scaler[ord(timeScale)];
            End;
  //  Else {hms : nothing to do};
  End;{case}
  EXCEPT;END;
End;


Function TQTimer.GetOptimalMeasure(measure:Int64; counter:TQTimerKind; var tScale:TQTimeScale):real;
var i:TQTimeScale;
    r:Real;
Begin
  TRY
    //hms as integer is simply returned as real :
    If counter = tkHMS Then
    Begin
      result := measure / 1;
      tScale := tsSeconds;
      Exit;
    end;

    For i := low(TQTimeScale) to high(TQTimeScale) Do
    Begin
      r := getTimeMeasureAs(i, measure, counter);
      If r > 100 Then break
      Else If (r < 99.9) And (r > 0.1) Then Break;
    End;
    result := r;
    tScale := i;
  EXCEPT;
    result := measure/1;
    //timeScale would be loosed...
  END;
End;


Procedure TQTimer.InitTimerEnvir;
// Retrieves & stores timers speeds ; Sets overhead, not resolutions.
Var i64In,i64Out,i64Sum : Int64;
    i: Integer;
    reg:TRegistry;
Begin
// queryPerformanceCounter
  If Not ( QueryPerformanceFrequency(FQpcFrq) ) Then
  Begin
    FQpcFrq := -1;
    FqpcOverHead := -1;
  End Else
  Begin
    i64Sum := 0;
    For i  := 0 To 19 Do   //try to average, despite not convincing...
    Begin
      QueryPerformanceCounter(i64In);
      QueryPerformanceCounter(i64Out);
      i64Sum := i64Sum +(i64Out-i64In);
    End;
    FqpcOverHead   := i64Sum div 20;
  End;
// Registry's stored frequency
  Reg := TRegistry.Create;
  FregFrq := -1;
  TRY
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.Access  := KEY_READ;
    If Reg.OpenKeyReadOnly('Hardware\Description\System\CentralProcessor\0') then
    Begin
      FRegFrq := Reg.ReadInteger('~MHz');
      FRegFrq := FRegFrq * 1000000;
    End;
  EXCEPT ;
  End;
// ReaDTimeStampCounter
  TRY
    i64In  := RDTSC;
    i64Out := RDTSC;
    FRdtOverhead   := i64Out - i64In;
    i64In  := RDTSC;
    i64Out := RDTSC;
    FRdtOverhead := Min((i64Out - i64In),FRdtOverhead);
    i64In  := RDTSC;
    i64Out := RDTSC;
    FRdtOverhead := Min((i64Out - i64In),FRdtOverhead);
  EXCEPT
    FRdtOverhead := -1;
  END;
  Self.FIsFinalized := (FqpcOverHead > -1) And (FRdtOverhead > -1);
End;

end.
